@@ -313,7 +313,7 @@ iter_instruction(
    uint i;
 
    if (inst->Instruction.Opcode == TGSI_OPCODE_END) {
-      if (ctx->index_of_END != ~0) {
+      if (ctx->index_of_END != ~0u) {
          report_error( ctx, "Too many END instructions" );
       }
       ctx->index_of_END = ctx->num_instructions;
@@ -454,7 +454,8 @@ iter_immediate(
     */
    if (imm->Immediate.DataType != TGSI_IMM_FLOAT32 &&
        imm->Immediate.DataType != TGSI_IMM_UINT32 &&
-       imm->Immediate.DataType != TGSI_IMM_INT32) {
+       imm->Immediate.DataType != TGSI_IMM_INT32 &&
+       imm->Immediate.DataType != TGSI_IMM_FLOAT64) {
       report_error( ctx, "(%u): Invalid immediate data type", imm->Immediate.DataType );
       return TRUE;
    }
@@ -485,7 +486,7 @@ epilog(
 
    /* There must be an END instruction somewhere.
     */
-   if (ctx->index_of_END == ~0) {
+   if (ctx->index_of_END == ~0u) {
       report_error( ctx, "Missing END instruction" );
    }
 
