@@ -7,13 +7,14 @@
 
 #include "venus-protocol/vn_protocol_renderer_queue.h"
 
+#include "vkr_context.h"
 #include "vkr_physical_device.h"
 #include "vkr_queue_gen.h"
 
 struct vkr_queue_sync *
 vkr_device_alloc_queue_sync(struct vkr_device *dev,
                             uint32_t fence_flags,
-                            uint64_t queue_id,
+                            uint32_t ring_idx,
                             uint64_t fence_id)
 {
    struct vn_device_proc_table *vk = &dev->proc_table;
@@ -56,7 +57,7 @@ vkr_device_alloc_queue_sync(struct vkr_device *dev,
 
    sync->device_lost = false;
    sync->flags = fence_flags;
-   sync->queue_id = queue_id;
+   sync->ring_idx = ring_idx;
    sync->fence_id = fence_id;
 
    return sync;
@@ -131,7 +132,7 @@ vkr_queue_sync_retire(struct vkr_context *ctx,
    struct vn_device_proc_table *vk = &dev->proc_table;
 
    if (vkr_renderer_flags & VKR_RENDERER_ASYNC_FENCE_CB) {
-      ctx->base.fence_retire(&ctx->base, sync->queue_id, sync->fence_id);
+      ctx->base.fence_retire(&ctx->base, sync->ring_idx, sync->fence_id);
       vkr_device_free_queue_sync(dev, sync);
    } else {
       vk->DestroyFence(dev->base.handle.device, sync->fence, NULL);
@@ -177,6 +178,9 @@ vkr_queue_destroy(struct vkr_context *ctx, struct vkr_queue *queue)
    list_del(&queue->busy_head);
    list_del(&queue->base.track_head);
 
+   if (queue->ring_idx > 0)
+      ctx->sync_queues[queue->ring_idx] = NULL;
+
    if (queue->base.id)
       vkr_context_remove_object(ctx, &queue->base);
    else
@@ -225,7 +229,7 @@ vkr_queue_thread(void *arg)
       list_del(&sync->head);
 
       if (vkr_renderer_flags & VKR_RENDERER_ASYNC_FENCE_CB) {
-         ctx->base.fence_retire(&ctx->base, sync->queue_id, sync->fence_id);
+         ctx->base.fence_retire(&ctx->base, sync->ring_idx, sync->fence_id);
          vkr_device_free_queue_sync(queue->device, sync);
       } else {
          list_addtail(&sync->head, &queue->signaled_syncs);
@@ -363,6 +367,26 @@ vkr_dispatch_vkGetDeviceQueue2(struct vn_dispatch_context *dispatch,
       return;
    }
 
+   const VkDeviceQueueTimelineInfoMESA *timeline_info = vkr_find_struct(
+      args->pQueueInfo->pNext, VK_STRUCTURE_TYPE_DEVICE_QUEUE_TIMELINE_INFO_MESA);
+   if (timeline_info) {
+      if (timeline_info->ringIdx == 0 ||
+          timeline_info->ringIdx >= ARRAY_SIZE(ctx->sync_queues)) {
+         vkr_log("invalid ring_idx %d", timeline_info->ringIdx);
+         vkr_cs_decoder_set_fatal(&ctx->decoder);
+         return;
+      }
+
+      if (ctx->sync_queues[timeline_info->ringIdx]) {
+         vkr_log("sync_queue %d already bound", timeline_info->ringIdx);
+         vkr_cs_decoder_set_fatal(&ctx->decoder);
+         return;
+      }
+
+      queue->ring_idx = timeline_info->ringIdx;
+      ctx->sync_queues[timeline_info->ringIdx] = queue;
+   }
+
    const vkr_object_id id =
       vkr_cs_handle_load_id((const void **)args->pQueue, VK_OBJECT_TYPE_QUEUE);
    vkr_queue_assign_object_id(ctx, queue, id);
@@ -401,6 +425,18 @@ vkr_dispatch_vkQueueWaitIdle(struct vn_dispatch_context *dispatch,
    vkr_cs_decoder_set_fatal(&ctx->decoder);
 }
 
+static void
+vkr_dispatch_vkQueueSubmit2(UNUSED struct vn_dispatch_context *dispatch,
+                            struct vn_command_vkQueueSubmit2 *args)
+{
+   struct vkr_queue *queue = vkr_queue_from_handle(args->queue);
+   struct vn_device_proc_table *vk = &queue->device->proc_table;
+
+   vn_replace_vkQueueSubmit2_args_handle(args);
+   args->ret =
+      vk->QueueSubmit2(args->queue, args->submitCount, args->pSubmits, args->fence);
+}
+
 static void
 vkr_dispatch_vkCreateFence(struct vn_dispatch_context *dispatch,
                            struct vn_command_vkCreateFence *args)
@@ -643,6 +679,9 @@ vkr_context_init_queue_dispatch(struct vkr_context *ctx)
    dispatch->dispatch_vkQueueSubmit = vkr_dispatch_vkQueueSubmit;
    dispatch->dispatch_vkQueueBindSparse = vkr_dispatch_vkQueueBindSparse;
    dispatch->dispatch_vkQueueWaitIdle = vkr_dispatch_vkQueueWaitIdle;
+
+   /* VK_KHR_synchronization2 */
+   dispatch->dispatch_vkQueueSubmit2 = vkr_dispatch_vkQueueSubmit2;
 }
 
 void
