@@ -700,7 +700,7 @@ enum vrend_tweak_type {
 #define VIRGL_LINK_SHADER_COMPUTE_HANDLE 6
 
 /* VIRGL_CCMD_CREATE_VIDEO_CODEC */
-#define VIRGL_CREATE_VIDEO_CODEC_SIZE       7
+#define VIRGL_CREATE_VIDEO_CODEC_MIN_SIZE   7
 #define VIRGL_CREATE_VIDEO_CODEC_HANDLE     1
 #define VIRGL_CREATE_VIDEO_CODEC_PROFILE    2
 #define VIRGL_CREATE_VIDEO_CODEC_ENTRYPOINT 3
@@ -708,9 +708,10 @@ enum vrend_tweak_type {
 #define VIRGL_CREATE_VIDEO_CODEC_LEVEL      5
 #define VIRGL_CREATE_VIDEO_CODEC_WIDTH      6
 #define VIRGL_CREATE_VIDEO_CODEC_HEIGHT     7
+#define VIRGL_CREATE_VIDEO_CODEC_MAX_REF    8
 
 /* VIRGL_CCMD_DESTROY_VIDEO_CODEC */
-#define VIRGL_DESTROY_VIDEO_CODEC_SIZE      1
+#define VIRGL_DESTROY_VIDEO_CODEC_MIN_SIZE  1
 #define VIRGL_DESTROY_VIDEO_CODEC_HANDLE    1
 
 /* VIRGL_CCMD_CREATE_VIDEO_BUFFER */
@@ -722,18 +723,18 @@ enum vrend_tweak_type {
 #define VIRGL_CREATE_VIDEO_BUFFER_RES_BASE  5
 
 /* VIRGL_CCMD_DESTROY_VIDEO_BUFFER */
-#define VIRGL_DESTROY_VIDEO_BUFFER_SIZE     1
+#define VIRGL_DESTROY_VIDEO_BUFFER_MIN_SIZE 1
 #define VIRGL_DESTROY_VIDEO_BUFFER_HANDLE   1
 
 /* VIRGL_CCMD_BEGIN_FRAME */
-#define VIRGL_BEGIN_FRAME_SIZE              2
+#define VIRGL_BEGIN_FRAME_MIN_SIZE          2
 #define VIRGL_BEGIN_FRAME_CDC_HANDLE        1
 #define VIRGL_BEGIN_FRAME_TGT_HANDLE        2
 
 /* VIRGL_CCMD_DECODE_MACROBLOCK */
 
 /* VIRGL_CCMD_DECODE_BITSTREAM */
-#define VIRGL_DECODE_BS_SIZE                5
+#define VIRGL_DECODE_BS_MIN_SIZE            5
 #define VIRGL_DECODE_BS_CDC_HANDLE          1
 #define VIRGL_DECODE_BS_TGT_HANDLE          2
 #define VIRGL_DECODE_BS_DSC_HANDLE          3
@@ -741,9 +742,15 @@ enum vrend_tweak_type {
 #define VIRGL_DECODE_BS_BUF_SIZE            5
 
 /* VIRGL_CCMD_ENCODE_BITSTREAM */
+#define VIRGL_ENCODE_BS_MIN_SIZE            5
+#define VIRGL_ENCODE_BS_CDC_HANDLE          1
+#define VIRGL_ENCODE_BS_SRC_HANDLE          2
+#define VIRGL_ENCODE_BS_DEST_HANDLE         3
+#define VIRGL_ENCODE_BS_DESC_HANDLE         4
+#define VIRGL_ENCODE_BS_FEED_HANDLE         5
 
 /* VIRGL_CCMD_END_FRAME */
-#define VIRGL_END_FRAME_SIZE                2
+#define VIRGL_END_FRAME_MIN_SIZE            2
 #define VIRGL_END_FRAME_CDC_HANDLE          1
 #define VIRGL_END_FRAME_TGT_HANDLE          2
 
