@@ -233,8 +233,7 @@ static int vrend_decode_clear_texture(struct vrend_context *ctx, const uint32_t
    arr[2] = get_buf_entry(buf, VIRGL_TEXTURE_ARRAY_C);
    arr[3] = get_buf_entry(buf, VIRGL_TEXTURE_ARRAY_D);
 
-   vrend_clear_texture(ctx, handle, level, &box, (void *) &arr);
-   return 0;
+   return vrend_clear_texture(ctx, handle, level, &box, (void *) &arr);
 }
 
 static int vrend_decode_set_viewport_state(struct vrend_context *ctx, const uint32_t *buf, uint32_t length)
@@ -673,6 +672,7 @@ static int vrend_decode_create_sampler_state(struct vrend_context *ctx, const ui
    state.compare_func = (tmp >> 16) & 0x7;
    state.seamless_cube_map = (tmp >> 19) & 0x1;
    state.max_anisotropy = (float)((tmp >> 20) & 0x3f);
+   state.normalized_coords = 0;
 
    state.lod_bias = uif(get_buf_entry(buf, VIRGL_OBJ_SAMPLER_STATE_LOD_BIAS));
    state.min_lod = uif(get_buf_entry(buf, VIRGL_OBJ_SAMPLER_STATE_MIN_LOD));
@@ -1641,7 +1641,7 @@ static int vrend_decode_create_video_codec(struct vrend_context *ctx,
 {
    struct vrend_video_context *vctx = vrend_context_get_video_ctx(ctx);
 
-   if (length != VIRGL_CREATE_VIDEO_CODEC_SIZE)
+   if (length < VIRGL_CREATE_VIDEO_CODEC_MIN_SIZE)
       return EINVAL;
 
    uint32_t handle     = get_buf_entry(buf, VIRGL_CREATE_VIDEO_CODEC_HANDLE);
@@ -1651,9 +1651,13 @@ static int vrend_decode_create_video_codec(struct vrend_context *ctx,
    uint32_t level      = get_buf_entry(buf, VIRGL_CREATE_VIDEO_CODEC_LEVEL);
    uint32_t width      = get_buf_entry(buf, VIRGL_CREATE_VIDEO_CODEC_WIDTH);
    uint32_t height     = get_buf_entry(buf, VIRGL_CREATE_VIDEO_CODEC_HEIGHT);
+   uint32_t max_ref    = 2; /* The max number of ref frames is 2 by default */
+
+   if (length >= VIRGL_CREATE_VIDEO_CODEC_MAX_REF)
+      max_ref = get_buf_entry(buf, VIRGL_CREATE_VIDEO_CODEC_MAX_REF);
 
    vrend_video_create_codec(vctx, handle, profile, entrypoint,
-                            chroma_fmt, level, width, height, 0);
+                            chroma_fmt, level, width, height, max_ref, 0);
 
    return 0;
 }
@@ -1664,7 +1668,7 @@ static int vrend_decode_destroy_video_codec(struct vrend_context *ctx,
 {
    struct vrend_video_context *vctx = vrend_context_get_video_ctx(ctx);
 
-    if (length != VIRGL_DESTROY_VIDEO_CODEC_SIZE)
+    if (length < VIRGL_DESTROY_VIDEO_CODEC_MIN_SIZE)
        return EINVAL;
 
    uint32_t handle = get_buf_entry(buf, VIRGL_DESTROY_VIDEO_CODEC_HANDLE);
@@ -1710,7 +1714,7 @@ static int vrend_decode_destroy_video_buffer(struct vrend_context *ctx,
 {
    struct vrend_video_context *vctx = vrend_context_get_video_ctx(ctx);
 
-   if (length != VIRGL_DESTROY_VIDEO_BUFFER_SIZE)
+   if (length < VIRGL_DESTROY_VIDEO_BUFFER_MIN_SIZE)
       return EINVAL;
 
    uint32_t handle = get_buf_entry(buf, VIRGL_DESTROY_VIDEO_BUFFER_HANDLE);
@@ -1725,7 +1729,7 @@ static int vrend_decode_begin_frame(struct vrend_context *ctx,
 {
    struct vrend_video_context *vctx = vrend_context_get_video_ctx(ctx);
 
-   if (length != VIRGL_BEGIN_FRAME_SIZE)
+   if (length < VIRGL_BEGIN_FRAME_MIN_SIZE)
       return EINVAL;
 
    uint32_t cdc_handle = get_buf_entry(buf, VIRGL_BEGIN_FRAME_CDC_HANDLE);
@@ -1741,7 +1745,7 @@ static int vrend_decode_decode_bitstream(struct vrend_context *ctx,
 {
    struct vrend_video_context *vctx = vrend_context_get_video_ctx(ctx);
 
-   if (length != VIRGL_DECODE_BS_SIZE)
+   if (length < VIRGL_DECODE_BS_MIN_SIZE)
       return EINVAL;
 
    uint32_t cdc_handle = get_buf_entry(buf, VIRGL_DECODE_BS_CDC_HANDLE);
@@ -1756,13 +1760,34 @@ static int vrend_decode_decode_bitstream(struct vrend_context *ctx,
    return 0;
 }
 
+static int vrend_decode_encode_bitstream(struct vrend_context *ctx,
+                                         const uint32_t *buf,
+                                         uint32_t length)
+{
+   struct vrend_video_context *vctx = vrend_context_get_video_ctx(ctx);
+
+   if (length < VIRGL_ENCODE_BS_MIN_SIZE)
+      return EINVAL;
+
+   uint32_t cdc_handle  = get_buf_entry(buf, VIRGL_ENCODE_BS_CDC_HANDLE);
+   uint32_t src_handle  = get_buf_entry(buf, VIRGL_ENCODE_BS_SRC_HANDLE);
+   uint32_t dest_handle = get_buf_entry(buf, VIRGL_ENCODE_BS_DEST_HANDLE);
+   uint32_t desc_handle = get_buf_entry(buf, VIRGL_ENCODE_BS_DESC_HANDLE);
+   uint32_t feed_handle = get_buf_entry(buf, VIRGL_ENCODE_BS_FEED_HANDLE);
+
+   vrend_video_encode_bitstream(vctx, cdc_handle, src_handle, dest_handle,
+                                desc_handle, feed_handle);
+
+   return 0;
+}
+
 static int vrend_decode_end_frame(struct vrend_context *ctx,
                                   const uint32_t *buf,
                                   uint32_t length)
 {
    struct vrend_video_context *vctx = vrend_context_get_video_ctx(ctx);
 
-   if (length != VIRGL_END_FRAME_SIZE)
+   if (length < VIRGL_END_FRAME_MIN_SIZE)
       return EINVAL;
 
    uint32_t cdc_handle = get_buf_entry(buf, VIRGL_END_FRAME_CDC_HANDLE);
@@ -1860,7 +1885,7 @@ static const vrend_decode_callback decode_table[VIRGL_MAX_COMMANDS] = {
    [VIRGL_CCMD_BEGIN_FRAME] = vrend_decode_begin_frame,
    [VIRGL_CCMD_DECODE_MACROBLOCK] = vrend_decode_dummy,
    [VIRGL_CCMD_DECODE_BITSTREAM] = vrend_decode_decode_bitstream,
-   [VIRGL_CCMD_ENCODE_BITSTREAM] = vrend_decode_dummy,
+   [VIRGL_CCMD_ENCODE_BITSTREAM] = vrend_decode_encode_bitstream,
    [VIRGL_CCMD_END_FRAME] = vrend_decode_end_frame,
 #else
    [VIRGL_CCMD_CREATE_VIDEO_CODEC] = vrend_unsupported,
@@ -1941,12 +1966,12 @@ static void vrend_decode_ctx_retire_fences(UNUSED struct virgl_context *ctx)
 
 static int vrend_decode_ctx_submit_fence(struct virgl_context *ctx,
                                          uint32_t flags,
-                                         uint64_t queue_id,
+                                         uint32_t ring_idx,
                                          uint64_t fence_id)
 {
    struct vrend_decode_ctx *dctx = (struct vrend_decode_ctx *)ctx;
 
-   if (queue_id)
+   if (ring_idx)
       return -EINVAL;
 
    return vrend_renderer_create_fence(dctx->grctx, flags, fence_id);
