@@ -83,6 +83,7 @@ struct vrend_fs_shader_info {
    int num_interps;
    int glsl_ver;
    bool has_sample_input;
+   bool has_noperspective;
    struct vrend_interp_info interpinfo[PIPE_MAX_SHADER_INPUTS];
 };
 
@@ -194,7 +195,7 @@ struct vrend_shader_key {
          uint32_t logicop_func : 4;
          uint32_t logicop_enabled : 1;
          uint32_t prim_is_points : 1;
-         uint32_t invert_origin : 1;
+         uint32_t lower_left_origin : 1;
          uint32_t available_color_in_bits : 4;
       } fs;
 
@@ -246,6 +247,8 @@ struct vrend_shader_cfg {
    uint32_t has_dual_src_blend : 1;
    uint32_t has_fbfetch_coherent : 1;
    uint32_t has_cull_distance : 1;
+   uint32_t has_nopersective : 1;
+   uint32_t has_texture_shadow_lod : 1;
 };
 
 struct vrend_context;
