@@ -22,7 +22,54 @@
  *
  **************************************************************************/
 
+/**
+ * @file
+ * The video implementation of the vrend renderer.
+ *
+ * It is based on the general virgl video submodule and handles data transfer
+ * and synchronization between host and guest.
+ *
+ * The relationship between vaSurface and video buffer objects:
+ *
+ *           GUEST (Mesa)           |       HOST (Virglrenderer)
+ *                                  |
+ *         +------------+           |          +------------+
+ *         | vaSurface  |           |          | vaSurface  | <------+
+ *         +------------+           |          +------------+        |
+ *               |                  |                                |
+ *  +---------------------------+   |   +-------------------------+  |
+ *  |    virgl_video_buffer     |   |   |    vrend_video_buffer   |  |
+ *  | +-----------------------+ |   |   |  +-------------------+  |  |
+ *  | |    vl_video_buffer    | |   |   |  | vrend_resource(s) |  |  |
+ *  | | +-------------------+ | |<--+-->|  +-------------------+  |  |
+ *  | | | virgl_resource(s) | | |   |   |  +--------------------+ |  |
+ *  | | +-------------------+ | |   |   |  | virgl_video_buffer |-+--+
+ *  | +-----------------------+ |   |   |  +--------------------+ |
+ *  +---------------------------+   |   +-------------------------+
+ *
+ * The relationship between vaContext and video codec objects:
+ *
+ *           GUEST (Mesa)         |         HOST (Virglrenderer)
+ *                                |
+ *         +------------+         |           +------------+
+ *         | vaContext  |         |           | vaContext  | <-------+
+ *         +------------+         |           +------------+         |
+ *               |                |                                  |
+ *  +------------------------+    |    +--------------------------+  |
+ *  |    virgl_video_codec   | <--+--> |    vrend_video_codec     |  |
+ *  +------------------------+    |    |  +--------------------+  |  |
+ *                                |    |  | virgl_video_codec  | -+--+
+ *                                |    |  +--------------------+  |
+ *                                |    +--------------------------+
+ *
+ * @author Feng Jiang <jiangfeng@kylinos.cn>
+ */
+
+
+#include <sys/param.h>
+
 #include "virgl_video.h"
+#include "virgl_video_hw.h"
 
 #include "vrend_debug.h"
 #include "vrend_winsys.h"
@@ -40,6 +87,8 @@ struct vrend_video_context {
 struct vrend_video_codec {
     struct virgl_video_codec *codec;
     uint32_t handle;
+    struct vrend_resource *feed_res;    /* encoding feedback */
+    struct vrend_resource *dest_res;    /* encoding coded buffer */
     struct vrend_video_context *ctx;
     struct list_head head;
 };
@@ -62,10 +111,16 @@ struct vrend_video_buffer {
     struct vrend_video_plane planes[3];
 };
 
+static struct vrend_video_codec *vrend_video_codec(
+        struct virgl_video_codec *codec)
+{
+    return virgl_video_codec_opaque_data(codec);
+}
+
 static struct vrend_video_buffer *vrend_video_buffer(
         struct virgl_video_buffer *buffer)
 {
-    return virgl_video_buffer_get_associated_data(buffer);
+    return virgl_video_buffer_opaque_data(buffer);
 }
 
 static struct vrend_video_codec *get_video_codec(
@@ -96,17 +151,18 @@ static struct vrend_video_buffer *get_video_buffer(
     return NULL;
 }
 
-static void flush_video_buffer(struct virgl_video_buffer *buffer,
-                               const struct virgl_video_dma_buf *dmabuf)
-{
-    EGLint img_attrs[16];
-    unsigned i, num_attrs;
-    struct vrend_resource *res;
-    struct vrend_video_plane *plane;
-    struct vrend_video_buffer *buf = vrend_video_buffer(buffer);
 
-    for (i = 0; i < dmabuf->num_planes && i < buf->num_planes; i++) {
-        plane = &buf->planes[i];
+static int sync_dmabuf_to_video_buffer(struct vrend_video_buffer *buf,
+                                       const struct virgl_video_dma_buf *dmabuf)
+{
+    if (!(dmabuf->flags & VIRGL_VIDEO_DMABUF_READ_ONLY)) {
+        vrend_printf("%s: dmabuf is not readable\n", __func__);
+        return -1;
+    }
+
+    for (unsigned i = 0; i < dmabuf->num_planes && i < buf->num_planes; i++) {
+        struct vrend_video_plane *plane = &buf->planes[i];
+        struct vrend_resource *res;
 
         res = vrend_renderer_ctx_res_lookup(buf->ctx->ctx, plane->res_handle);
         if (!res) {
@@ -116,20 +172,15 @@ static void flush_video_buffer(struct virgl_video_buffer *buffer,
 
         /* dmabuf -> eglimage */
         if (EGL_NO_IMAGE_KHR == plane->egl_image) {
-            num_attrs = 0;
-            img_attrs[num_attrs++] = EGL_LINUX_DRM_FOURCC_EXT;
-            img_attrs[num_attrs++] = dmabuf->planes[i].drm_format;
-            img_attrs[num_attrs++] = EGL_WIDTH;
-            img_attrs[num_attrs++] = dmabuf->width / (i + 1);
-            img_attrs[num_attrs++] = EGL_HEIGHT;
-            img_attrs[num_attrs++] = dmabuf->height / (i + 1);
-            img_attrs[num_attrs++] = EGL_DMA_BUF_PLANE0_FD_EXT;
-            img_attrs[num_attrs++] = dmabuf->planes[i].fd;
-            img_attrs[num_attrs++] = EGL_DMA_BUF_PLANE0_OFFSET_EXT;
-            img_attrs[num_attrs++] = dmabuf->planes[i].offset;
-            img_attrs[num_attrs++] = EGL_DMA_BUF_PLANE0_PITCH_EXT;
-            img_attrs[num_attrs++] = dmabuf->planes[i].pitch;
-            img_attrs[num_attrs++] = EGL_NONE;
+            EGLint img_attrs[16] = {
+                EGL_LINUX_DRM_FOURCC_EXT,       dmabuf->planes[i].drm_format,
+                EGL_WIDTH,                      dmabuf->width / (i + 1),
+                EGL_HEIGHT,                     dmabuf->height / (i + 1),
+                EGL_DMA_BUF_PLANE0_FD_EXT,      dmabuf->planes[i].fd,
+                EGL_DMA_BUF_PLANE0_OFFSET_EXT,  dmabuf->planes[i].offset,
+                EGL_DMA_BUF_PLANE0_PITCH_EXT,   dmabuf->planes[i].pitch,
+                EGL_NONE
+            };
 
             plane->egl_image = eglCreateImageKHR(eglGetCurrentDisplay(),
                     EGL_NO_CONTEXT, EGL_LINUX_DMA_BUF_EXT, NULL, img_attrs);
@@ -150,7 +201,7 @@ static void flush_video_buffer(struct virgl_video_buffer *buffer,
         glFramebufferTexture2D(GL_READ_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,
                                GL_TEXTURE_2D, plane->texture, 0);
 
-        /* framebuffer -> surface */
+        /* framebuffer -> vrend_video_buffer.planes[i] */
         glBindTexture(GL_TEXTURE_2D, res->id);
         glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0,
                             res->base.width0, res->base.height0);
@@ -158,14 +209,161 @@ static void flush_video_buffer(struct virgl_video_buffer *buffer,
 
     glBindTexture(GL_TEXTURE_2D, 0);
     glBindFramebuffer(GL_FRAMEBUFFER, 0);
+
+    return 0;
 }
 
+static int sync_video_buffer_to_dmabuf(struct vrend_video_buffer *buf,
+                                       const struct virgl_video_dma_buf *dmabuf)
+{
+    if (!(dmabuf->flags & VIRGL_VIDEO_DMABUF_WRITE_ONLY)) {
+        vrend_printf("%s: dmabuf is not writable\n", __func__);
+        return -1;
+    }
+
+    for (unsigned i = 0; i < dmabuf->num_planes && i < buf->num_planes; i++) {
+        struct vrend_video_plane *plane = &buf->planes[i];
+        struct vrend_resource *res;
+
+        res = vrend_renderer_ctx_res_lookup(buf->ctx->ctx, plane->res_handle);
+        if (!res) {
+            vrend_printf("%s: res %d not found\n", __func__, plane->res_handle);
+            continue;
+        }
+
+        /* dmabuf -> eglimage */
+        if (EGL_NO_IMAGE_KHR == plane->egl_image) {
+            EGLint img_attrs[16] = {
+                EGL_LINUX_DRM_FOURCC_EXT,       dmabuf->planes[i].drm_format,
+                EGL_WIDTH,                      dmabuf->width / (i + 1),
+                EGL_HEIGHT,                     dmabuf->height / (i + 1),
+                EGL_DMA_BUF_PLANE0_FD_EXT,      dmabuf->planes[i].fd,
+                EGL_DMA_BUF_PLANE0_OFFSET_EXT,  dmabuf->planes[i].offset,
+                EGL_DMA_BUF_PLANE0_PITCH_EXT,   dmabuf->planes[i].pitch,
+                EGL_NONE
+            };
+
+            plane->egl_image = eglCreateImageKHR(eglGetCurrentDisplay(),
+                    EGL_NO_CONTEXT, EGL_LINUX_DMA_BUF_EXT, NULL, img_attrs);
+        }
+
+        if (EGL_NO_IMAGE_KHR == plane->egl_image) {
+            vrend_printf("%s: create egl image failed\n", __func__);
+            continue;
+        }
+
+        /* eglimage -> texture */
+        glBindTexture(GL_TEXTURE_2D, plane->texture);
+        glEGLImageTargetTexture2DOES(GL_TEXTURE_2D,
+                                    (GLeglImageOES)(plane->egl_image));
+
+        /* vrend_video_buffer.planes[i] -> framebuffer */
+        glBindFramebuffer(GL_READ_FRAMEBUFFER, plane->framebuffer);
+        glFramebufferTexture2D(GL_READ_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,
+                               GL_TEXTURE_2D, res->id, 0);
+
+        /* framebuffer -> texture */
+        glBindTexture(GL_TEXTURE_2D, plane->texture);
+        glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0,
+                            res->base.width0, res->base.height0);
+
+    }
+
+    glBindTexture(GL_TEXTURE_2D, 0);
+    glBindFramebuffer(GL_FRAMEBUFFER, 0);
+
+    return 0;
+}
+
+
+static void vrend_video_decode_completed(
+                                struct virgl_video_codec *codec,
+                                const struct virgl_video_dma_buf *dmabuf)
+{
+    struct vrend_video_buffer *buf = vrend_video_buffer(dmabuf->buf);
+
+    (void)codec;
+
+    sync_dmabuf_to_video_buffer(buf, dmabuf);
+}
+
+
+static void vrend_video_enocde_upload_picture(
+                                struct virgl_video_codec *codec,
+                                const struct virgl_video_dma_buf *dmabuf)
+{
+    struct vrend_video_buffer *buf = vrend_video_buffer(dmabuf->buf);
+
+    (void)codec;
+
+    sync_video_buffer_to_dmabuf(buf, dmabuf);
+}
+
+static void vrend_video_encode_completed(
+                                struct virgl_video_codec *codec,
+                                const struct virgl_video_dma_buf *src_buf,
+                                const struct virgl_video_dma_buf *ref_buf,
+                                unsigned num_coded_bufs,
+                                const void * const *coded_bufs,
+                                const unsigned *coded_sizes)
+{
+    void *buf;
+    unsigned i, size, data_size;
+    struct virgl_video_encode_feedback feedback;
+    struct vrend_video_codec *cdc = vrend_video_codec(codec);
+
+    (void)src_buf;
+    (void)ref_buf;
+
+    if (!cdc->dest_res || !cdc->feed_res)
+        return;
+
+    memset(&feedback, 0, sizeof(feedback));
+
+    /* sync coded data to guest */
+    if (has_bit(cdc->dest_res->storage_bits, VREND_STORAGE_GL_BUFFER)) {
+        glBindBufferARB(cdc->dest_res->target, cdc->dest_res->id);
+        buf = glMapBufferRange(cdc->dest_res->target, 0,
+                               cdc->dest_res->base.width0, GL_MAP_WRITE_BIT);
+        for (i = 0, data_size = 0; i < num_coded_bufs &&
+                    data_size < cdc->dest_res->base.width0; i++) {
+            size = MIN(cdc->dest_res->base.width0 - data_size, coded_sizes[i]);
+            memcpy((uint8_t *)buf + data_size, coded_bufs[i], size);
+            vrend_write_to_iovec(cdc->dest_res->iov, cdc->dest_res->num_iovs,
+                                 data_size, coded_bufs[i], size);
+            data_size += size;
+        }
+        glUnmapBuffer(cdc->dest_res->target);
+        glBindBufferARB(cdc->dest_res->target, 0);
+        feedback.stat = VIRGL_VIDEO_ENCODE_STAT_SUCCESS;
+        feedback.coded_size = data_size;
+    } else {
+        vrend_printf("unexcepted coded res type\n");
+        feedback.stat = VIRGL_VIDEO_ENCODE_STAT_FAILURE;
+        feedback.coded_size = 0;
+    }
+
+    /* send feedback */
+    vrend_write_to_iovec(cdc->feed_res->iov, cdc->feed_res->num_iovs,
+                         0, (char *)(&feedback),
+                         MIN(cdc->feed_res->base.width0, sizeof(feedback)));
+
+    cdc->dest_res = NULL;
+    cdc->feed_res = NULL;
+}
+
+static struct virgl_video_callbacks video_callbacks = {
+    .decode_completed           = vrend_video_decode_completed,
+    .encode_upload_picture      = vrend_video_enocde_upload_picture,
+    .encode_completed           = vrend_video_encode_completed,
+};
+
 int vrend_video_init(int drm_fd)
 {
     if (drm_fd < 0)
         return -1;
 
-    return virgl_video_init(drm_fd, flush_video_buffer, 0);
+    return virgl_video_init(drm_fd, &video_callbacks, 0);
 }
 
 void vrend_video_fini(void)
@@ -186,6 +384,7 @@ int vrend_video_create_codec(struct vrend_video_context *ctx,
                              uint32_t level,
                              uint32_t width,
                              uint32_t height,
+                             uint32_t max_ref,
                              uint32_t flags)
 {
     struct vrend_video_codec *cdc = get_video_codec(ctx, handle);
@@ -218,7 +417,9 @@ int vrend_video_create_codec(struct vrend_video_context *ctx,
     args.level = level;
     args.width = width;
     args.height = height;
+    args.max_references = max_ref;
     args.flags = flags;
+    args.opaque = cdc;
     cdc->codec = virgl_video_create_codec(&args);
     if (!cdc->codec) {
         free(cdc);
@@ -279,12 +480,12 @@ int vrend_video_create_buffer(struct vrend_video_context *ctx,
     args.width = width;
     args.height = height;
     args.interlaced = 0;
+    args.opaque = buf;
     buf->buffer = virgl_video_create_buffer(&args);
     if (!buf->buffer) {
         free(buf);
         return -1;
     }
-    virgl_video_buffer_set_associated_data(buf->buffer, buf);
 
     for (i = 0; i < ARRAY_SIZE(buf->planes); i++)
         buf->planes[i].egl_image = EGL_NO_IMAGE_KHR;
@@ -495,8 +696,9 @@ int vrend_video_decode_bitstream(struct vrend_video_context *ctx,
         vrend_printf("%s: desc res %d not found\n", __func__, desc_handle);
         goto err;
     }
-    vrend_read_from_iovec(res->iov, res->num_iovs,
-                          0, (char *)(&desc), sizeof(desc));
+    memset(&desc, 0, sizeof(desc));
+    vrend_read_from_iovec(res->iov, res->num_iovs, 0, (char *)(&desc),
+                          MIN(res->base.width0, sizeof(desc)));
     modify_picture_desc(cdc, tgt, &desc);
 
     err = virgl_video_decode_bitstream(cdc->codec, tgt->buffer, &desc,
@@ -509,6 +711,50 @@ err:
     return err;
 }
 
+int vrend_video_encode_bitstream(struct vrend_video_context *ctx,
+                                 uint32_t cdc_handle,
+                                 uint32_t src_handle,
+                                 uint32_t dest_handle,
+                                 uint32_t desc_handle,
+                                 uint32_t feed_handle)
+{
+    union virgl_picture_desc desc;
+    struct vrend_resource *dest_res, *desc_res, *feed_res;
+    struct vrend_video_codec  *cdc = get_video_codec(ctx, cdc_handle);
+    struct vrend_video_buffer *src = get_video_buffer(ctx, src_handle);
+
+    if (!cdc || !src)
+        return -1;
+
+    /* Feedback resource */
+    feed_res = vrend_renderer_ctx_res_lookup(ctx->ctx, feed_handle);
+    if (!feed_res) {
+        vrend_printf("%s: feedback res %d not found\n", __func__, feed_handle);
+        return -1;
+    }
+
+    /* Picture descriptor resource */
+    desc_res = vrend_renderer_ctx_res_lookup(ctx->ctx, desc_handle);
+    if (!desc_res) {
+        vrend_printf("%s: desc res %d not found\n", __func__, desc_handle);
+        return -1;
+    }
+    memset(&desc, 0, sizeof(desc));
+    vrend_read_from_iovec(desc_res->iov, desc_res->num_iovs, 0, (char *)(&desc),
+                          MIN(desc_res->base.width0, sizeof(desc)));
+
+    /* Destination buffer resource. */
+    dest_res = vrend_renderer_ctx_res_lookup(ctx->ctx, dest_handle);
+    if (!dest_res) {
+        vrend_printf("%s: dest res %d not found\n", __func__, dest_handle);
+        return -1;
+    }
+
+    cdc->feed_res = feed_res;
+    cdc->dest_res = dest_res;
+
+    return virgl_video_encode_bitstream(cdc->codec, src->buffer, &desc);
+}
 
 int vrend_video_end_frame(struct vrend_video_context *ctx,
                           uint32_t cdc_handle,
